/*
 * GREAT-ER
 *
 * INIT.H
 *
 * Initialization of global variables
 *
 * author =  Geert Boeije
 * state =   07.01.1999
*/

#ifndef INIT_H
#define INIT_H

/* first, include some other stuff */

#include <stdio.h>
#include "def.h"
#include "params.h"

/* globals */

extern char SimulationTitle[150];
extern char WorkDirectory[500];

extern int FloatSize;
extern int IntSize;

extern int NrEmissionSubmodels;
extern int NrTransTreatSubmodels;
extern int NrSubmodels;

extern int ModelClass[];
extern int NrSub[];
extern int NrDestinations[];
extern int Destination[NR_PROCESSES][MAX_NR_DESTINATIONS][2];
extern int DestNrBypass[];
extern int NrParamsInProcess[];
extern int NrParamsOnlyRiver;
extern int NrParamsDischarge;

extern int OneStateVarSize;
extern int AllStateVarSize;
extern int AmatrixSize;
extern int AsubMatrixSize;

extern long AllIfaceSize;
extern long AllParamSize;
extern long AllSummarySize;

extern int NrSegments;
extern int NrSegmentsOnlyRiver;
extern int NrSegmentsDischarge;
extern int NrEmissions;
extern int NrRivClasses;
extern int NrWWTPs;

extern int NrWWTPExceptions;
extern int NrRiverExceptions;


extern int* SegmentDone;

extern int* SegmentParOffset;
extern int ParametersArrayOffset[];
extern int StateVarsArrayOffset[];
extern int* AsubMatrixOffset;

extern int NrDistriParams[];
extern int NrMonteCarloShots;
extern int* NonGeoRefDistriParIndex;
extern int* NonGeoRefDistriType;
extern int* ChemicalDistriParIndex;
extern int* ChemicalDistriType;
extern int* DischargeDistriParIndex;
extern int* DischargeDistriType;
extern int* RiverDistriParIndex;
extern int* RiverDistriType;
extern int* EmissionDistriType;
extern int* EmissionDistriParIndex;
extern int* RivClassDistriType;
extern int* RivClassDistriParIndex;
extern int* WWTPDistriType;
extern int* WWTPDistriParIndex;
extern int* SummaryDistriParIndex;
extern int* SummaryDistriType;
extern int TotNonGeoRefDistriPar;
extern int TotChemicalDistriPar;
extern int TotDischargeDistriPar;
extern int TotRiverDistriPar;
extern int TotSummaryDistriPar;
extern int TotEmissionDistriPar;
extern int TotWWTPDistriPar;
extern int TotRivClassDistriPar;

extern int WWTP_ID_Location;
extern int Emission_ID_Location;

extern NonGeoRef_ParPtr NonGeoRefParams;
extern Chemical_ParPtr  ChemicalParams;
extern FloatPtr         Distri_NonGeoRefParams;
extern FloatPtr         Distri_ChemicalParams;
extern FloatPtr         Distri_RiverParams;
extern FloatPtr         Distri_DischargeParams;
extern FloatPtr         Distri_EmissionParams;
extern FloatPtr         Distri_RivClassParams;
extern FloatPtr         Distri_WWTPParams;
extern FloatPtr         EmissionData;
extern FloatPtr         WWTPData;
extern FloatPtr         CSewer_Q_DWF;
extern FloatPtr         Distri_SummaryVars;
extern FloatPtr         WWTPExceptions;
extern FloatPtr         RiverExceptions;
extern Map_Ptr          Map;

extern FloatPtr A;
extern FloatPtr B;
extern FloatPtr X;

extern FILE* OutputFile;
extern FILE* ErrorFile;
extern FILE* LogFile;

extern char NonGeoRefFileName[500];
extern char ChemicalFileName[500];
extern char RiverFileName[500];
extern char DischargeFileName[500];
extern char EmissionFileName[500];
extern char WWTPFileName[500];
extern char RivClassFileName[500];
extern char DistriFileName[500];
extern char ErrorFileName[500];
extern char OutputFileName[500];
extern char ResultsFileName[500];
extern char INIFileName[500];
extern char WWTPExceptionsFileName[500];
extern char RiverExceptionsFileName[500];


extern int LogCtrlParams;
extern int LogShotParameters;
extern int LogShotResults;
extern int LogSummaryResults;

extern int LogToFile;
extern int ShowMessages;
extern int ShowErrorMessages;
extern int Visible;

extern int ComplexityModeRiver;
extern int ComplexityModeWWTP;
extern int ComplexityModeSewer;
extern int ComplexityModeEmission;
extern int ImposeComplexityModeRiver;
extern int ImposeComplexityModeWWTP;
extern int ImposeComplexityModeSewer;
extern int UseExceptions;
extern int ErrorFree;
extern int AllFinished;

extern char C[100];


/* function prototypes */

void Initialize (void);

/***************************************************************************************/
/***************************************************************************************/
/***************************************************************************************/
#ifdef TEST_MODE
extern FILE* TestLog;
#endif  
/***************************************************************************************/
/***************************************************************************************/
/***************************************************************************************/

#endif
