/*
 * GREAT-ER
 *
 * MTCARLO.H
 *
 * Monte Carlo Engine
 *
 * author =  Geert Boeije
 * state =   21.03.1997
*/

#ifndef MTCARLO_H
#define MTCARLO_H

typedef struct
{
  FloatType mean, sdev;
}
NormalParams;

FloatType Ran2 ( long* idum_arg );

FloatType RandomNumber ( void );

NormalParams LogNormalFromNormal ( FloatType mean_arg,
                                   FloatType sdev_arg );

NormalParams NormalFromLogNormal ( FloatType mean_arg,
                                   FloatType sdev_arg );

FloatType StDevFrom95thPercentile ( FloatType mean_arg,
                                    FloatType low95_arg );

FloatType PercentileFromNormal ( FloatType mean_arg,
                                 FloatType sdev_arg,
                                 FloatType perc_arg );

FloatType PercentileFromLogNormal ( FloatType mean_arg,
                                    FloatType sdev_arg,
                                    FloatType perc_arg );

FloatType GasDev ( void );

FloatType ShotFromUniform( FloatType mean_arg,
                           FloatType halfwidth_arg );

FloatType ShotFromNormal ( FloatType mean_arg,
                           FloatType sdev_arg );

FloatType ShotFromLogNormal ( FloatType mean_arg,
                              FloatType sdev_arg );

FloatType ShotFromFlowType ( FloatType mean_arg,
                             FloatType sdev_arg );

FloatType ObtainShotFromDistribution ( FloatPtr distri_par_arg,
                                       int distri_type_arg );

void MakeMonteCarloVector ( FloatPtr from_arg,
                            FloatPtr to_arg,
                            int nr_arg,
                            int* distri_type_arg,
                            int* distri_index_arg    );

FloatType CorrShotFromNormal ( FloatType mean_arg,
                               FloatType sdev_arg,
                               FloatType corr_arg );

FloatType CorrShotFromLogNormal ( FloatType mean_arg,
                                  FloatType sdev_arg,
                                  FloatType corr_arg );

FloatType ObtainCorrelatedShot ( FloatPtr distri_par_arg,
                                 int distri_type_arg,
                                 FloatType corr_arg );

void GenerateMonteCarloShot ( FloatPtr allsegments_par_arg,
                              FloatPtr allsegments_rivclass_arg,
                              FloatType* flow_perc_arg );

void ModifyDistribVector ( FloatPtr vector_arg,
                           int nr_arg,
                           int* distri_type_arg,
                           int* distri_index_arg    );

void InitializeFlowTypeDistrib ( void );

void MakeSums ( FloatPtr allsegments_summary_arg,
                FloatPtr allsegments_sums_arg );

void CalculateDistribution ( FloatPtr allsegments_sums_arg );

#endif

