/*
 * GREAT-ER
 *
 * DATA_OUT.C
 *
 * data output section
 *
 * author =  Geert Boeije
 *
 * changes by Intevation, http://intevation.net
 * state =   16.02.2000
 *
 * Intevation:
 *	replaced IN and OUT by IN__ and OUT__
 * 	bug fix, see below
*/

#include <stdio.h>
#include <string.h>
#include "init.h"
#include "names.h"
#include "error.h"
#include "usriface.h"
#include "navigate.h"
#include "data_out.h"

/*
 * ---------------------------------------------------------------------
 * verify the specified work directory
 * ---------------------------------------------------------------------
*/

int CheckWorkDirectory ( void )
{
  int l;
  FILE* dummy_file;
  int err = 0;
  char f[1000];

  if ( WorkDirectory[0] != '\0' )
  {
    l = strlen (WorkDirectory) - 1;

    /* is there a / or \ behind the directory name ? */

    /* on UNIX systems, a forward slash is used as directory separator */

    #ifdef UNIX

    if ( WorkDirectory[l] != '/' )
    {
      WorkDirectory[l+1] = '/' ;
      WorkDirectory[l+2] = '\0' ;
    }

    /* on PC systems, a backward slash is used as directory separator */

    #else

    if ( WorkDirectory[l] != '\\' )
    {
      WorkDirectory[l+1] = '\\';
      WorkDirectory[l+2] = '\0';
    }

    #endif

    /* test the directory's availability by writing and deleting a test file ! */

    strcpy ( f, WorkDirectory );
    strcat ( f, "00000000.000" );

    if ( ! (dummy_file = fopen(f, "wt")) )
    {
      err = 1;
    }
    else
    {
      fclose ( dummy_file );
      remove ( f );
    }
  }

  return err;
}


/*
 * ---------------------------------------------------------------------
 * open file for output
 * ---------------------------------------------------------------------
*/

int OpenOutputFile ( FILE** source_arg,
                     char*  filename_arg )
{
  char f[1000];

  strcpy ( f, WorkDirectory );
  strcat ( f, filename_arg );

  /* open the file */

  if ( (*source_arg = fopen(f, "wt")) == NULL)
  {
    GenerateErrorMessage (100, f);
    return 0;
  }
  else
    return 1;
}

/*
 * ---------------------------------------------------------------------
 * install error file pointer
 * ---------------------------------------------------------------------
*/

int InstallErrorFile (void)
{
  int test;
  int err = 0;
  char c[500];

  /* first check the work directory */

  err = CheckWorkDirectory ();

  if ( err )
  {
    strcpy ( c, WorkDirectory );
    WorkDirectory[0] = '\0';
  }

  /* open error log file stream */

  test = OpenOutputFile ( &ErrorFile, ErrorFileName );

  if ( test && !err )
    return 1;
  else
  {
    if ( err )
      GenerateErrorMessage ( 108, c );
    return 0;
  }
}

/*
 * ---------------------------------------------------------------------
 * install output file pointers
 * ---------------------------------------------------------------------
*/

int InstallOutputFiles (void)
{
  /* open output file stream */

  if (!OpenOutputFile ( &OutputFile, OutputFileName ))
    return 0;

  /* open results log file stream */

  if ( LogToFile )
  {
    if (!OpenOutputFile ( &LogFile, ResultsFileName ))
      return 0;
  }
  else
    LogFile = NULL;

/*+++++++++++++++++++++++++++++++++++++++++
  +++++++++++++++++++++++++++++++++++++++++*/
#ifdef TEST_MODE
OpenOutputFile ( &TestLog, "testlog.txt" );
#endif  
/*+++++++++++++++++++++++++++++++++++++++++
  +++++++++++++++++++++++++++++++++++++++++*/

  /* success ! */

  return 1;
}

/*
 * ---------------------------------------------------------------------
 * close output file pointers
 * ---------------------------------------------------------------------
*/

void CloseOutputFiles (void)
{
  if ( OutputFile ) fclose ( OutputFile );
  if ( LogFile ) fclose ( LogFile );
  if ( ErrorFile ) fclose ( ErrorFile );
#ifdef TEST_MODE
if ( TestLog ) fclose ( TestLog );
#endif
}


/*
 * ---------------------------------------------------------------------
 * write control parameters to log device
 * ---------------------------------------------------------------------
*/

void LogControlParams ( FloatType first_stretch_arg,
                        long      tot_alloc_arg )
{
  #ifdef GUI_MODE
  if ( !LogCtrlParams || !LogToFile )
    return;
  #else
  if ( !LogCtrlParams )
    return;
  #endif

  WriteLog ("GREAT-ER Simulation\n\n");
  WriteLog ("Simulator version "VERSION_NUMBER" ("VERSION_DATE")\n");

  sprintf ( C, "%s\n\n*************************************\n\n", SimulationTitle);
  WriteLog ( C );

  WriteLog ("Control Parameters:\n\nInput Files:\n");

  sprintf ( C, " Work Directory =         \"%s\"\n\n", WorkDirectory );
  WriteLog ( C );

  sprintf ( C, " NON-GEOREF FILE =        \"%s\"\n", NonGeoRefFileName );
  WriteLog ( C );
  sprintf ( C, " CHEMICAL FILE =          \"%s\"\n", ChemicalFileName );
  WriteLog ( C );
  sprintf ( C, " RIVER FILE =             \"%s\"\n", RiverFileName );
  WriteLog ( C );
  sprintf ( C, " RIVER CLASS FILE =       \"%s\"\n", RivClassFileName );
  WriteLog ( C );
  sprintf ( C, " DISCHARGE FILE  =        \"%s\"\n", DischargeFileName );
  WriteLog ( C );
  sprintf ( C, " WWTP FILE =              \"%s\"\n", WWTPFileName );
  WriteLog ( C );
  sprintf ( C, " EMISSION FILE =          \"%s\"\n\n", EmissionFileName );
  WriteLog ( C );
  sprintf ( C, " WWTP EXCEPTIONS FILE =   \"%s\"\n\n", WWTPExceptionsFileName );
  WriteLog ( C );
  sprintf ( C, " RIVER EXCEPTIONS FILE =  \"%s\"\n\n", RiverExceptionsFileName );
  WriteLog ( C );
  sprintf ( C, " DISTRIBUTION TYPE FILE = \"%s\"\n", DistriFileName );
  WriteLog ( C );
  sprintf ( C, " ERROR FILE =             \"%s\"\n", ErrorFileName );
  WriteLog ( C );
  sprintf ( C, " OUTPUT FILE =            \"%s\"\n", OutputFileName );
  WriteLog ( C );
  sprintf ( C, " RESULTS FILE =           \"%s\"\n\n", ResultsFileName );
  WriteLog ( C );

  sprintf ( C, "USE SITE-SPECIFIC EXCEPTIONS = %d\n", UseExceptions );
  WriteLog ( C );

  WriteLog ("\nGeography:\n");

  sprintf ( C, " NR SEGMENTS      = %d\n", NrSegments );
  WriteLog ( C );
  sprintf ( C, " NR DISCHARGES    = %d\n", NrSegmentsDischarge );
  WriteLog ( C );
  sprintf ( C, " NR WWTPs         = %d\n", NrWWTPs );
  WriteLog ( C );
  sprintf ( C, " NR RIVER CLASSES = %d\n", NrRivClasses );
  WriteLog ( C );
  sprintf ( C, " NR EMISSION DATA   = %d\n", NrEmissions );
  WriteLog ( C );

  WriteLog ("\nSimulation:\n");

  sprintf ( C, " NR SHOTS        = %d\n", NrMonteCarloShots );
  WriteLog ( C );
  sprintf ( C, " FIRST STRETCH   = %f\n", first_stretch_arg );
  WriteLog ( C );
  sprintf ( C, " DEFAULT COMPLEXITY MODE RIVER = %d\n", ComplexityModeRiver );
  WriteLog ( C );
  sprintf ( C, " DEFAULT COMPLEXITY MODE WWTP  = %d\n", ComplexityModeWWTP );
  WriteLog ( C );
  sprintf ( C, " DEFAULT COMPLEXITY MODE SEWER = %d\n", ComplexityModeSewer );
  WriteLog ( C );
  sprintf ( C, " IMPOSE DEFAULT COMPLEXITY MODE RIVER = %d\n", ImposeComplexityModeRiver );
  WriteLog ( C );
  sprintf ( C, " IMPOSE DEFAULT COMPLEXITY MODE WWTP  = %d\n", ImposeComplexityModeWWTP );
  WriteLog ( C );
  sprintf ( C, " IMPOSE DEFAULT COMPLEXITY MODE SEWER = %d\n", ImposeComplexityModeSewer );
  WriteLog ( C );

  WriteLog ("\nOutput:\n");

  sprintf ( C, " LOG CONTROL PARAMS  = %d\n", LogCtrlParams );
  WriteLog ( C );
  sprintf ( C, " LOG SHOT PARAMETERS = %d\n", LogShotParameters );
  WriteLog ( C );
  sprintf ( C, " LOG SHOT RESULTS    = %d\n", LogShotResults );
  WriteLog ( C );
  sprintf ( C, " LOG SUMMARY RESULTS = %d\n", LogSummaryResults );
  WriteLog ( C );
  WriteLog ( "\n" );
  sprintf ( C, " LOG TO FILE         = %d\n", LogToFile );
  WriteLog ( C );
  sprintf ( C, " OPERATION VISIBLE   = %d\n", Visible );
  WriteLog ( C );
  sprintf ( C, " SHOW MESSAGES       = %d\n", ShowMessages );
  WriteLog ( C );
  sprintf ( C, " SHOW ERROR MESSAGES = %d\n", ShowErrorMessages );
  WriteLog ( C );

  sprintf ( C, "\nTotal amount of allocated memory = %.2f Kilobytes\n\n", 
            tot_alloc_arg/1024.0 );
  WriteLog ( C );

  sprintf ( C, "*************************************\n\n" );
  WriteLog ( C );

}

/*
 * ---------------------------------------------------------------------
 * write segment results to log device
 * ---------------------------------------------------------------------
*/

void LogSegmentResults ( FloatType segment_ID_arg,
                         FloatPtr  segment_iface_arg,
                         FloatPtr  segment_summary_arg,
                         int       start_proc_arg )
{
  int process, submodel, st_var, term;
  int summaries;
  static int first_time = 1;



  #ifdef GUI_MODE
  if ( !LogShotResults || !LogToFile )
    return;
  #else
  if ( !LogShotResults )
    return;
  #endif

  if ( first_time )
  {
    first_time = 0;

    WriteLog ( "***********************\n" );
    WriteLog ( "INDIVIDUAL SHOT RESULTS\n" );
    WriteLog ( "***********************\n\n" );
  }

  /* write to log which segment is being simulated */

  sprintf ( C, "SEGMENT %f\n", segment_ID_arg );
  WriteLog ( C );

  /* STEP 1 = write detailed results for all processes and submodels */

  for ( process=start_proc_arg ; process<NR_PROCESSES ; ++process )
  {
    /* write to log which process is being simulated */

    for ( submodel=0 ; submodel<NrSub[process] ; ++submodel )
    {
      sprintf ( C, "\nprocess: %s%s%s\n\n", ProcessName[process],
                                          " - " ,
                                          SubmodelName[process][submodel] );
      WriteLog ( C );
      
      for ( st_var=0 ; st_var<NR_STATEVARS ; ++st_var )
      {
        sprintf (C," %15s ", StateVarName[st_var]);
        WriteLog ( C );

        for ( term=0 ; term<(ModelClass[process] == EM ? 1 : 2) ; ++term )
        {
          sprintf ( C, "%10s = %11.5f %s",
                       TerminalName[term],
                       X[ oX(st_var,process,term,submodel) ],
                       StateVarUnit[st_var] );
          WriteLog ( C );
        }
        WriteLog ( "\n" );
      }
    }
  }

  /* STEP 2 = write interface variables */

  sprintf ( C, "\n\nINTERFACE VARIABLES FOR SEGMENT %f\n\n", segment_ID_arg );
  WriteLog ( C );

  /*+++++++++++++++++++++++++++++++++++++++++
    +++++++++++++++++++++++++++++++++++++++++*/
  #ifdef TEST_MODE
  fprintf ( TestLog, "%f\t", segment_ID_arg );
  #endif  
  /*+++++++++++++++++++++++++++++++++++++++++
    +++++++++++++++++++++++++++++++++++++++++*/

  for ( st_var=0 ; st_var<NR_STATEVARS ; ++st_var )
  {
    sprintf (C," %15s ", StateVarName[st_var]);
    WriteLog ( C );

    for ( term=0 ; term<NR_IFACE_TERMS ; ++term )
    {
      sprintf ( C, "%10s = %11.5f %s", 
                   TerminalName[term],
                   segment_iface_arg[ st_var*NR_IFACE_TERMS + term ],
                   StateVarUnit[st_var] );
      WriteLog ( C );
    }
    WriteLog ("\n");
  }

  /*+++++++++++++++++++++++++++++++++++++++++
    +++++++++++++++++++++++++++++++++++++++++*/
  #ifdef TEST_MODE
  fprintf ( TestLog, "%f\t%f\n", 
            segment_iface_arg[ _Q_*NR_IFACE_TERMS + IN__ ], 
            segment_iface_arg[ _Q_*NR_IFACE_TERMS + OUT__ ] );
  #endif  
  /*+++++++++++++++++++++++++++++++++++++++++
    +++++++++++++++++++++++++++++++++++++++++*/


  /* STEP 3 = write summary variables */

  sprintf ( C, "\n\nSUMMARY FOR SEGMENT %f\n\n", segment_ID_arg );
  WriteLog ( C );

  if ( start_proc_arg == RIVER )
    summaries = START_OF_DISCHARGE;
  else
    summaries = NR_SUMMARY_TERMS;
  

  for ( st_var=0 ; st_var<NR_SUMMARY_VARS ; ++st_var )
  {
    for ( term=0 ; term<summaries ; ++term )
    {
      sprintf ( C, "  %20s %s = %11.5f %s\n", 
                   SummaryTerminalName[term],
                   SummaryVarName[st_var], 
                   segment_summary_arg[ oS( st_var, term) ],
                   SummaryVarUnit[st_var] );
      WriteLog ( C );
    }
    WriteLog ("\n");
  }

  WriteLog ("-----------------------------------\n\n");

}

/*
 * ---------------------------------------------------------------------
 * write distributions to log device
 * ---------------------------------------------------------------------
*/

void LogSummaryDistribs ( FloatPtr allsegments_par_arg )
{
  General_ParPtr segment_par;
  FloatPtr current_distri_seg;
  FloatPtr current_distri;
  int st_var, term, segment, i;
  int where;

  #ifdef GUI_MODE
  if ( !LogSummaryResults || !LogToFile )
    return;
  #else
  if ( !LogSummaryResults )
    return;
  #endif

  WriteLog ( "*********************\n" );
  WriteLog ( "RESULTS DISTRIBUTIONS\n" );
  WriteLog ( "*********************\n\n" );

  for ( segment=0 ; segment<NrSegments ; ++segment )
  {
    segment_par = (General_ParPtr)(&allsegments_par_arg[ SegmentParOffset[ segment ] ]);

    if ( (int)(segment_par->segment_type) == THE_SEA )
      continue;

    /* DISTRIBUTIONS */

    current_distri_seg = &Distri_SummaryVars[ segment * TotSummaryDistriPar ];

    sprintf ( C, "DISTRIBUTION OF SUMMARY VARIABLES FOR SEGMENT %f\n\n", 
              segment_par->segment_ID );
    WriteLog ( C );

    for ( st_var=0 ; st_var<NR_SUMMARY_VARS ; ++st_var )
    {
      for ( term=0 ; term<NR_SUMMARY_TERMS ; ++term )
      {
        if ( !(int)(segment_par->discharge) && term >= START_OF_DISCHARGE )
          continue;

        where = st_var * NR_SUMMARY_TERMS + term;
        current_distri = current_distri_seg + SummaryDistriParIndex[ where ];

        sprintf ( C, "  %20s %s (%s)  ", SummaryTerminalName[term],
                                         SummaryVarName[st_var], 
                                         SummaryVarUnit[st_var] );
        WriteLog ( C );

        for ( i=0 ; i<NrDistriParams[ SummaryDistriType[ where ] ] ; ++i )
        {
          sprintf ( C, "%s %14.8f  ",
                       DistriParamName[ SummaryDistriType[ where ] ][ i ],
                       current_distri[ i ] );
          WriteLog ( C );
        }
        WriteLog ("\n");
      }
    }
    WriteLog ("\n\n");
  }

  WriteLog ("-----------------------------------\n\n");

}


/*
 * ---------------------------------------------------------------------
 * write parameters shot to log device
 * ---------------------------------------------------------------------
*/

void LogParameters ( FloatPtr allsegments_par_arg,
                     FloatPtr allsegments_rivclass_arg,
                     FloatType flow_perc_arg )
{
  FloatPtr here;
  River_ParPtr river;
  RivClass_ParPtr rc;
  WWTP_ParPtr stp;
  PrimParPtr stp1;
  ASParPtr stp2;
  TFParPtr stp3;
  int segment;
  int emission_nr;
  int i;

  #ifdef GUI_MODE
  if ( !LogShotParameters || !LogToFile )
    return;
  #else
  if ( !LogShotParameters )
    return;
  #endif

  WriteLog ( "******\n" );
  if ( flow_perc_arg != -1 )
    sprintf ( C, "\nSHOT // Flow Percentile = %f\n\n", flow_perc_arg );
  else
    sprintf ( C, "\nSINGLE SHOT // Mean Flow\n\n" );
  WriteLog ( C );
  WriteLog ( "******\n" );


  /* non-georef */

  WriteLog ( "-----------------------------\n");
  WriteLog ( "Non-geo-referenced parameters\n");
  WriteLog ( "-----------------------------\n\n");
  

  WriteLog ( "\n  general (environmental) information\n");
  WriteLog (   "  -----------------------------------\n\n");

  sprintf ( C, "groundwater conserv. = %f\n\n", 
            NonGeoRefParams->groundwater_conservative);
  WriteLog ( C );


  sprintf ( C, "   water temperature = %f\n", NonGeoRefParams->temp_water);
  WriteLog ( C );
  sprintf ( C, "  sewage temperature = %f\n", NonGeoRefParams->temp_sewage);
  WriteLog ( C );
  sprintf ( C, "     air temperature = %f\n", NonGeoRefParams->temp_air);
  WriteLog ( C );
  sprintf ( C, "          wind speed = %f\n", NonGeoRefParams->wind_speed);
  WriteLog ( C );
  sprintf ( C, "   air mixing height = %f\n", NonGeoRefParams->h_air);
  WriteLog ( C );

  WriteLog ( "\n  sewage properties\n");
  WriteLog (   "  -----------------\n\n");

  sprintf ( C, "    suspended solids = %f\n", NonGeoRefParams->SS_sewage);
  WriteLog ( C );
  sprintf ( C, "         solids f_oc = %f\n", NonGeoRefParams->f_oc_sewage);
  WriteLog ( C );
  sprintf ( C, "      solids density = %f\n", NonGeoRefParams->ro_SS_sewage);
  WriteLog ( C );

  sprintf ( C, "    per cap BOD flux = %f\n", NonGeoRefParams->PHI_cap_BOD);
  WriteLog ( C );
  sprintf ( C, "      per cap N flux = %f\n", NonGeoRefParams->PHI_cap_N);
  WriteLog ( C );

  WriteLog ( "\n  primary settler properties\n");
  WriteLog (   "  --------------------------\n\n");

  sprintf ( C, "         BOD removal = %f\n", NonGeoRefParams->R_prim_BOD);
  WriteLog ( C );
  sprintf ( C, "           N removal = %f\n", NonGeoRefParams->R_prim_N);
  WriteLog ( C );

  WriteLog ( "\n  AS properties\n");
  WriteLog (   "  -------------\n\n");

  sprintf ( C, "    end. respiration = %f\n", NonGeoRefParams->r_end);
  WriteLog ( C );
  sprintf ( C, "      ML solids f_oc = %f\n", NonGeoRefParams->f_oc_ML);
  WriteLog ( C );
  sprintf ( C, "   ML solids density = %f\n", NonGeoRefParams->ro_SS_ML);
  WriteLog ( C );
  sprintf ( C, "surface aeration PSI = %f\n", NonGeoRefParams->PSI);
  WriteLog ( C );
  sprintf ( C, "             kGa/kLa = %f\n", NonGeoRefParams->kGa_kLa);
  WriteLog ( C );
  sprintf ( C, "         ML salinity = %f\n", NonGeoRefParams->ML_salinity);
  WriteLog ( C );

  WriteLog ( "\n  combined sewer\n");
  WriteLog (   "  --------------\n\n");

  sprintf ( C, "          CS Q / DWF = river class specific values\n");
  WriteLog ( C );
  
  WriteLog ( "\n  several defaults\n");
  WriteLog (   "  ----------------\n\n");

  sprintf ( C, "   mass transfer air = %f\n", NonGeoRefParams->K_air);
  WriteLog ( C );
  sprintf ( C, " mass transfer water = %f\n", NonGeoRefParams->K_water);
  WriteLog ( C );
  sprintf ( C, " prim. sorption rate = %f\n", NonGeoRefParams->k_sorb_prim);
  WriteLog ( C );
  sprintf ( C, "    ML sorption rate = %f\n", NonGeoRefParams->k_sorb_ML);
  WriteLog ( C );
  sprintf ( C, "  sec. sorption rate = %f\n", NonGeoRefParams->k_sorb_sec);
  WriteLog ( C );
  sprintf ( C, "settler efficiency 1 = %f\n", NonGeoRefParams->a_prim);
  WriteLog ( C );
  sprintf ( C, "settler efficiency 2 = %f\n", NonGeoRefParams->a_sec);
  WriteLog ( C );

  WriteLog ( "\n  correlations with flow\n");
  WriteLog (   "  ----------------------\n\n");


  sprintf ( C, "    corr temperature = %f\n", NonGeoRefParams->temp_corr);
  WriteLog ( C );
  sprintf ( C, "     corr wind speed = %f\n", NonGeoRefParams->wind_corr);
  WriteLog ( C );
  sprintf ( C, "             corr SS = %f\n", NonGeoRefParams->SS_corr);
  WriteLog ( C );
  sprintf ( C, "             corr DO = %f\n", NonGeoRefParams->DO_corr);
  WriteLog ( C );
  sprintf ( C, "            corr BOD = %f\n", NonGeoRefParams->BOD_corr);
  WriteLog ( C );
  sprintf ( C, "           corr k_SS = %f\n", NonGeoRefParams->k_SS_corr);
  WriteLog ( C );
  WriteLog ( "\n" );

  

  /* chemical */

  WriteLog ( "-----------------------------\n");
  WriteLog ( "Chemical parameters\n");
  WriteLog ( "-----------------------------\n\n");

  WriteLog ( "\n  chemical properties\n");
  WriteLog (   "  -------------------\n\n");

  WriteLog ( "* phys / chem\n\n");

  sprintf ( C, "                  MM = %f\n", ChemicalParams->MM);
  WriteLog ( C );
  sprintf ( C, "                K_ow = %f\n", ChemicalParams->K_ow);
  WriteLog ( C );
  sprintf ( C, "     Vapour Pressure = %f\n", ChemicalParams->P_vapour);
  WriteLog ( C );
  sprintf ( C, "          solubility = %f\n", ChemicalParams->solubility);
  WriteLog ( C );
  sprintf ( C, "                 pKa = %f\n", ChemicalParams->pKa);
  WriteLog ( C );
  sprintf ( C, "                   a = %f\n", ChemicalParams->a);
  WriteLog ( C );

  WriteLog ("\n");

  sprintf ( C, "            Kd river = %f\n", ChemicalParams->Kd_river);
  WriteLog ( C );
  sprintf ( C, "           Kd sewage = %f\n", ChemicalParams->Kd_sewage);
  WriteLog ( C );
  sprintf ( C, "     Kd mixed liquor = %f\n", ChemicalParams->Kd_ML);
  WriteLog ( C );
  sprintf ( C, "                   H = %f\n", ChemicalParams->H);
  WriteLog ( C );

  WriteLog ( "\n* biodegradation\n\n");

  sprintf ( C, "   k_standard_biodeg = %f\n", ChemicalParams->k_std_biodeg);
  WriteLog ( C );
  sprintf ( C, "         a_anaerobic = %f\n", ChemicalParams->a_anaerobic);
  WriteLog ( C );
  sprintf ( C, "            a_anoxic = %f\n", ChemicalParams->a_anoxic);
  WriteLog ( C );
  sprintf ( C, "                K_DO = %f\n", ChemicalParams->K_DO);
  WriteLog ( C );
  sprintf ( C, "            a_sorbed = %f\n", ChemicalParams->a_sorbed);
  WriteLog ( C );
  sprintf ( C, "                 Q10 = %f\n", ChemicalParams->Q10);
  WriteLog ( C );
  
  WriteLog ( "\n" );

  sprintf ( C, " WWTP Monod kinetics = %f\n", ChemicalParams->use_Monod);
  WriteLog ( C );
  sprintf ( C, "         Monod k_max = %f\n", ChemicalParams->k_max);
  WriteLog ( C );
  sprintf ( C, "             Monod Y = %f\n", ChemicalParams->Y);
  WriteLog ( C );
  sprintf ( C, "           Monod K_s = %f\n", ChemicalParams->K_s);
  WriteLog ( C );
  sprintf ( C, "             Monod b = %f\n", ChemicalParams->b);
  WriteLog ( C );

  WriteLog ( "\n  River model\n");
  WriteLog (   "  -----------\n\n");

  WriteLog ( "* mode 1\n\n");

  sprintf ( C, "          k_instream = %f\n", ChemicalParams->k_instream);
  WriteLog ( C );

  WriteLog ( "\n* mode 2\n\n");

  sprintf ( C, "               k_vol = %f\n", ChemicalParams->k_vol);
  WriteLog ( C );
  sprintf ( C, "               k_deg = %f\n", ChemicalParams->k_deg);
  WriteLog ( C );

  WriteLog ( "\n* mode 3\n\n");

  sprintf ( C, "              m3_vol = %f\n", ChemicalParams->m3_vol);
  WriteLog ( C );
  sprintf ( C, "              m3_sed = %f\n", ChemicalParams->m3_sed);
  WriteLog ( C );
  sprintf ( C, "          m3_deg_bio = %f\n", ChemicalParams->m3_deg_bio);
  WriteLog ( C );
  sprintf ( C, "        m3_deg_photo = %f\n", ChemicalParams->m3_deg_photo);
  WriteLog ( C );
  sprintf ( C, "         m3_deg_hydr = %f\n", ChemicalParams->m3_deg_hydr);
  WriteLog ( C );

  sprintf ( C, "            k_hydr_n = %f\n", ChemicalParams->k_hydr_n);
  WriteLog ( C );
  sprintf ( C, "            k_hydr_a = %f\n", ChemicalParams->k_hydr_a);
  WriteLog ( C );
  sprintf ( C, "            k_hydr_b = %f\n", ChemicalParams->k_hydr_b);
  WriteLog ( C );
  sprintf ( C, "             k_photo = %f\n", ChemicalParams->k_photo);
  WriteLog ( C );
  
  sprintf ( C, "             a_river = %f\n", ChemicalParams->a_river);
  WriteLog ( C );


  WriteLog ( "\n  WWTP model\n");
  WriteLog (   "  ----------\n\n");

  WriteLog ( "* mode 1\n\n");

  sprintf ( C, "           R_primary = %f\n", ChemicalParams->R_primary);
  WriteLog ( C );
  sprintf ( C, "  R_activated_sludge = %f\n", ChemicalParams->R_activated_sludge);
  WriteLog ( C );
  sprintf ( C, "  R_trickling_filter = %f\n", ChemicalParams->R_trickling_filter);
  WriteLog ( C );

  WriteLog ( "\n* mode 2,3\n\n");

  sprintf ( C, "                a_AS = %f\n", ChemicalParams->a_AS);
  WriteLog ( C );
  sprintf ( C, "       k_WWTP_nonbio = %f\n", ChemicalParams->k_WWTP_nonbio);
  WriteLog ( C );

  WriteLog ( "\n  Sewer model\n");
  WriteLog (   "  -----------\n\n");

  sprintf ( C, "             R_sewer = %f\n", ChemicalParams->R_sewer);
  WriteLog ( C );



  /* river classes */

  WriteLog ( "\n\n-------------\n");
  WriteLog ( "River Classes\n");
  WriteLog ( "-------------\n\n");


  for ( i=0 ; i<NrRivClasses ; ++i )
  {
    rc = (RivClass_ParPtr)&(allsegments_rivclass_arg[ NR_RIVCLASS_PARAMS * i ]);

    sprintf ( C, "River Class ID = %f\n\n", rc->RivClass_ID);
    WriteLog ( C );
    sprintf ( C, "Selected Mode = %f\n\n", rc->mode);
    WriteLog ( C );

    sprintf ( C, "                lake = %f\n", rc->lake);
    WriteLog ( C );
    sprintf ( C, "flow corr river-WWTP = %f\n", rc->corr_riv_csewer);
    WriteLog ( C );

    sprintf ( C, "      CS flow / DWF  = %f\n", CSewer_Q_DWF[i]);
    WriteLog ( C );

    sprintf ( C, "          a_rivclass = %f\n", rc->a_rivclass);
    WriteLog ( C );


    WriteLog ( "  \nmode 2 data\n\n" );
    sprintf ( C, "          default SS = %f\n", rc->SS);
    WriteLog ( C );
    sprintf ( C, "    SS settling rate = %f\n", rc->k_SS);
    WriteLog ( C );
    sprintf ( C, "                f_oc = %f\n", rc->f_oc);
    WriteLog ( C );
    sprintf ( C, "        v_sed_growth = %f\n", rc->v_sed_growth);
    WriteLog ( C );
    sprintf ( C, "             eps_sed = %f\n", rc->eps_sed);
    WriteLog ( C );
    sprintf ( C, "           ro_SS_sed = %f\n", rc->ro_SS_sed);
    WriteLog ( C );

    WriteLog ( "  \nmode 3 data\n\n" );
    sprintf ( C, "           default X = %f\n", rc->X);
    WriteLog ( C );
    sprintf ( C, "          default DO = %f\n", rc->DO);
    WriteLog ( C );
    sprintf ( C, "         default BOD = %f\n", rc->BOD);
    WriteLog ( C );
    sprintf ( C, "          default pH = %f\n", rc->pH);
    WriteLog ( C );
    sprintf ( C, "     light ext. rate = %f\n", rc->k_light);
    WriteLog ( C );

    WriteLog ("\n");
  }

  /* WWTPs */

  WriteLog ( "\n----------------------------\n");
  WriteLog ( "Waste Water Treatment Plants\n");
  WriteLog ( "----------------------------\n\n");

  for ( i=0 ; i<NrWWTPs ; ++i )
  {
    stp = (WWTP_ParPtr)&(WWTPData[ NR_WWTP_PARAMS * i ]);

    sprintf ( C, "WWTP ID = %f\n\n", stp->WWTP_ID);
    WriteLog ( C );
    sprintf ( C, "Selected Mode = %f\n\n", stp->mode);
    WriteLog ( C );

    stp1 = (PrimParPtr)stp;
    stp2 = (ASParPtr)stp;
    stp3 = (TFParPtr)stp;

    switch ( (int)stp->type )
    {
      case PRIMARY:

        sprintf ( C, "      Treatment Type = PRIMARY TREATMENT\n");
        WriteLog ( C );
        sprintf ( C, "      capacity / DWF = %f\n", stp1->capacity_prim);
        WriteLog ( C );
        sprintf ( C, "       settler depth = %f\n", stp1->h_prim);
        WriteLog ( C );
        sprintf ( C, "       overflow rate = %f\n", stp1->v_overflow_prim);
        WriteLog ( C );
        sprintf ( C, "            R_SS_max = %f\n", stp1->R_SS_prim_max);
        WriteLog ( C );

        break;

      case ACTIVATED_SLUDGE:

        sprintf ( C, "      Treatment Type = ACTIVATED SLUDGE\n");
        WriteLog ( C );

        if ( (int) stp2->primary_present )
        {
          WriteLog ("  Primary Settler:\n");

          sprintf ( C, "      capacity / DWF = %f\n", stp1->capacity_prim);
          WriteLog ( C );
          sprintf ( C, "       settler depth = %f\n", stp2->h_prim);
          WriteLog ( C );
          sprintf ( C, "       overflow rate = %f\n", stp2->v_overflow_prim);
          WriteLog ( C );
          sprintf ( C, "            R_SS_max = %f\n", stp2->R_SS_prim_max);
          WriteLog ( C );
        }
        else
          WriteLog ("  Primary Settler: none\n");

        WriteLog ("  Secondary Settler:\n");
        sprintf ( C, "       settler depth = %f\n", stp2->h_sec);
        WriteLog ( C );
        sprintf ( C, "       overflow rate = %f\n", stp2->v_overflow_sec);
        WriteLog ( C );
        sprintf ( C, "            R_SS_max = %f\n", stp2->R_SS_sec_max);
        WriteLog ( C );

        WriteLog ("  Activated Sludge System:\n");

        sprintf ( C, "      capacity / DWF = %f\n", stp2->capacity_sec);
        WriteLog ( C );
        sprintf ( C, "  people equivalents = %f\n", stp2->nr_PE);
        WriteLog ( C );
        sprintf ( C, "     mixed liquor SS = %f\n", stp2->SS_ML);
        WriteLog ( C );
        sprintf ( C, "sludge recycle ratio = %f\n", stp2->recycle);
        WriteLog ( C );
        sprintf ( C, "    biomass activity = %f\n", stp2->a_biomass);
        WriteLog ( C );
        sprintf ( C, "          tank depth = %f\n", stp2->h_act_sludge);
        WriteLog ( C );
        sprintf ( C, "                 B_X = %f\n", stp2->B_X);
        WriteLog ( C );
        sprintf ( C, " DO in aeration tank = %f\n", stp2->DO);
        WriteLog ( C );
        sprintf ( C, "       aeration type = %s\n", 
                            ( stp2->aer_type == 1 ? "surface" : "bubble" ) );
        WriteLog ( C );
        sprintf ( C, "           f_aerobic = %f\n", stp2->f_aerobic);
        WriteLog ( C );
        sprintf ( C, "            f_anoxic = %f\n", stp2->f_anoxic);
        WriteLog ( C );
        sprintf ( C, "         f_anaerobic = %f\n", stp2->f_anaerobic);
        WriteLog ( C );
        
        sprintf ( C, "  nitrif. efficiency = %f\n", stp2->eps_N);
        WriteLog ( C );
        sprintf ( C, "denitrif. efficiency = %f\n", stp2->eps_DN);
        WriteLog ( C );

        break;

      case TRICKLING_FILTER:

        sprintf ( C, "      Treatment Type = TRICKLING FILTER\n");
        WriteLog ( C );
        sprintf ( C, "                       Mode 1 model only");
        WriteLog ( C );
        if ( (int) stp3->primary_present )
          WriteLog ( "                       Primary Settler is present\n");

        break;

      default:

        sprintf ( C, "      Treatment Type = INVALID\n");
        WriteLog ( C );
        break;
    }
    WriteLog ("\n");
  }

  /* georef */

  WriteLog ( "\n-------------------------\n");
  WriteLog ( "Geo-Referenced Parameters\n");
  WriteLog ( "-------------------------\n\n");

  for ( segment=0 ; segment<NrSegments ; ++segment )
  {
    here = &allsegments_par_arg[ SegmentParOffset [segment] ];
    river = (River_ParPtr)(&here[ ParametersArrayOffset[RIVER] ]);

    if ( (int)( ((General_ParPtr)here)->segment_type ) == THE_SEA )
      continue;


    sprintf ( C, "\nPARAMETERS FOR SEGMENT %f\n\n", 
              ((General_ParPtr)here)->segment_ID );
    WriteLog ( C );

    sprintf ( C, "segment type = %f\n", ((General_ParPtr)here)->segment_type );
    WriteLog ( C );    
    sprintf ( C, "  upstream 1 = %f\n", ((General_ParPtr)here)->upstream_one );
    WriteLog ( C );    
    sprintf ( C, "  upstream 2 = %f\n", ((General_ParPtr)here)->upstream_two );
    WriteLog ( C );    
    sprintf ( C, " river class = %f\n", ((General_ParPtr)here)->rivclass );
    WriteLog ( C );    

    WriteLog ( "\nRiver Parameters\n\n");

    sprintf ( C, "        Q = %f\n", river->Q);
    WriteLog ( C );    
    sprintf ( C, "        v = %f\n", river->v);
    WriteLog ( C );    
    sprintf ( C, "   length = %f\n", river->length);
    WriteLog ( C );
    sprintf ( C, "    depth = %f\n", river->depth);
    WriteLog ( C );
    sprintf ( C, "       SS = %f\n", river->SS);
    WriteLog ( C );
    sprintf ( C, "       DO = %f\n", river->DO);
    WriteLog ( C );
    sprintf ( C, "      BOD = %f\n", river->BOD);
    WriteLog ( C );
    sprintf ( C, "       pH = %f\n", river->pH);
    WriteLog ( C );
    sprintf ( C, "        X = %f\n", river->X);
    WriteLog ( C );
    WriteLog ("\n");

    if ( (int)(((General_ParPtr)here)->discharge) )
    {
      emission_nr = (&Map[ segment ])->emission;
      WriteLog ("\n");
      LogDischargeParameters ( (FloatPtr)here, 
                               ( emission_nr != -1 ? 
                                 (EmissionParPtr )&(EmissionData[ NR_EMISSION_PARAMS * emission_nr ]) 
                                 : 0 ) );
    }

    WriteLog ("\n\n");
  }
}      

void LogDischargeParameters ( FloatPtr par_arg,
                              EmissionParPtr em_arg )
{
  int i,j;

  DomesticEm_ParPtr dom =
    (DomesticEm_ParPtr)(&par_arg[ ParametersArrayOffset[DOM] ]);

  NonDomesticEm_ParPtr nondom =
    (NonDomesticEm_ParPtr)(&par_arg[ ParametersArrayOffset[NONDOM] ]);

  RunoffEm_ParPtr runoff =
    (RunoffEm_ParPtr)(&par_arg[ ParametersArrayOffset[RUNOFF] ]);

  /*
  OnSiteTreatment_ParPtr onsite =
    (OnSiteTreatment_ParPtr)(&par_arg[ ParametersArrayOffset[ONSITE] ]);
  */

  Sewer_ParPtr sewer =
    (Sewer_ParPtr)(&par_arg[ ParametersArrayOffset[SEWER] ]);

  Treatment_ParPtr treat =
    (Treatment_ParPtr)(&par_arg[ ParametersArrayOffset[TREAT] ]);

/*
  SmallSurf_ParPtr surfwat =
    (SmallSurf_ParPtr)(&par_arg[ ParametersArrayOffset[SURFWAT] ]);
*/

  WriteLog ( "Waste Water Pathway Parameters\n\n" );

  /* emission */

  sprintf ( C, "          emission ID = %f\n\n", em_arg->emission_ID );
  WriteLog ( C );

  /* domestic emission */

  WriteLog ( "domestic emission\n\n" );

  /*LogOutflow ( (FloatPtr)dom->out, DOM );*/

  sprintf ( C, "          inhabitants = %f\n", dom->inhabitants );
  WriteLog ( C );
  sprintf ( C, "     dry weather flow = %f\n", dom->dry_weather_flow );
  WriteLog ( C );

  if ( em_arg )
  {
    sprintf ( C, "  product consumption = %f\n", em_arg->consumption );
    WriteLog ( C );
  }

  WriteLog ("\n");

  /* nondomestic emission */

  WriteLog ( "non-domestic emission\n\n" );

  #if NR_DEST_NONDOM > 1
  LogOutflow ( (FloatPtr)nondom->out, NONDOM );
  #endif

  for ( i=0 ; i<NR_SUB_NONDOM ; ++i )
  {
    sprintf ( C,   "  nondom flow[%d] = %f\n",i,nondom->flow[i] );
    WriteLog( C );

    if ( em_arg )
    {
      sprintf ( C, "  nondom flux = %f\n", em_arg->nondom );
      WriteLog ( C );
    }
  }

  WriteLog ("\n");

  /* runoff */

  WriteLog ( "runoff\n\n" );

  #if NR_DEST_RUNOFF > 1
  LogOutflow ( (FloatPtr)runoff->out, RUNOFF );
  #endif

  for ( i=0 ; i<NR_SUB_RUNOFF ; ++i )
  {
    sprintf ( C,   "  runoff flow[%d] = %f\n",i,runoff->flow[i] );
    WriteLog( C );

    if ( em_arg )
    {
      sprintf ( C, "  runoff flux = %f\n", em_arg->runoff );
      WriteLog ( C );
    }
  }

  WriteLog ("\n");

  /* on-site-treatment */

  WriteLog ( "on-site treatment\n\n" );

  #if NR_DEST_RUNOFF > 1
  LogOutflow ( (FloatPtr)onsite->out, ONSITE );
  WriteLog ("\n");
  #endif

  /* sewer */

  WriteLog ( "sewer transport\n\n" );

  #if NR_DEST_SEWER > 1
  LogOutflow ( (FloatPtr)sewer->out, SEWER );
  WriteLog ("\n");
  #endif

  /* waste water treatment */

  WriteLog ( "waste water treatment\n\n" );

  #if NR_DEST_TREAT > 1
  LogOutflow ( (FloatPtr)treat->out, TREAT );
  #endif

  for ( j=0 ; j<NR_WWTP_CLASSES ; ++j )
    {
      sprintf ( C, "      WWTP class [%2d] = %f\tfraction = %f\n", 
                j, treat->WWTP[j][0], treat->WWTP[j][1]);
      WriteLog( C );
    }

  /* small surface water */

  /* ground water */
}

void LogOutflow ( FloatPtr out_arg,
                  int process_arg )
{
  int i,j;

  for ( i=0 ; i<NrSub[ process_arg ] ; ++i )
    for ( j=0 ; j<NrDestinations[ process_arg ] - 1 ; ++j )
    {
      sprintf ( C, "  out p[%d] -> dest[%d] = %f\n",
                i, j+1, out_arg[ i*(NrDestinations[ process_arg ]-1) + j ]);
      WriteLog ( C );
    }
}



/*
 ***********************************************************************
 * 
 * THIS IS THE IMPORTANT ONE
 *
 * WRITE OUTPUT FILE, TO BE READ BY THE GIS
 *
 ***********************************************************************
*/

void WriteOutputFile ( FloatPtr allsegments_par_arg )
{
  General_ParPtr segment_par;
  FloatPtr current_distri_seg;
  FloatPtr current_distri;
  int st_var, term, segment, i;
  int where;

  for ( segment=0 ; segment<NrSegments ; ++segment )
  {
    segment_par = (General_ParPtr)(&allsegments_par_arg[ SegmentParOffset[ segment ] ]);

    /* go to next segment if this segment is a hypothetical one */

    if ( (int)(segment_par->segment_type) == THE_SEA )
      continue;

    /* this segment is ok */

    current_distri_seg = &Distri_SummaryVars[ segment * TotSummaryDistriPar ];

    /* write segment ID */

    fprintf ( OutputFile, "%f\t", segment_par->segment_ID );

    for ( st_var=0 ; st_var<NR_SUMMARY_VARS ; ++st_var )
    {
      for ( term=0 ; term<NR_SUMMARY_TERMS ; ++term )
      {
        where = st_var * NR_SUMMARY_TERMS + term;
        current_distri = &current_distri_seg[ SummaryDistriParIndex[ where ] ];

        for ( i=0 ; i<NrDistriParams[ SummaryDistriType[ where ] ] ; ++i )
	/* Intevation: replaced %f by %g for output of very small numbers
		in exponential representation instead of "0.0000" */
          fprintf ( OutputFile, "%g\t", current_distri[ i ] );
      }
    }

    fprintf (OutputFile, "\n" );
  }
}
