/*
 * GREAT-ER
 *
 * GUI.C
 *
 * author =  Geert Boeije
 * state =   28.05.1998
*/

#include "init.h"

#ifdef GUI_MODE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include "extgui.h"
#include "usriface.h"

int Change = 0;
int ExitCode = 0;

static HINSTANCE hInstance;

HWND HMainWindow;
HFONT HMainWinFont;
HWND HWindow[2];
HFONT HWinFont[2];

char DoneBarTxt[100] = "0 % finished";

long MainWinStyle =   WS_OVERLAPPED;


/******************************************************
 *                                                    *
 * Main Window Functions                              *
 *                                                    *
 ******************************************************
*/

LRESULT CALLBACK WndProc( HWND hwnd,
                          UINT message,
                          WPARAM wParam,
                          LPARAM lParam)
{
  HDC hdc;
  PAINTSTRUCT ps;  

  switch (message) 
  {
    case WM_PAINT:
      hdc = BeginPaint(hwnd, &ps);
      paint(hdc);
      EndPaint(hwnd, &ps);
      return 0;

    case WM_DESTROY:
      PostQuitMessage(0);
      return 0;
  }

  return DefWindowProc(hwnd, message, wParam, lParam);
}

void InitMainWindow ( HINSTANCE hInst,
                      HINSTANCE hPrevInstance, 
                      LPSTR lpszCmdParam, 
                      int nCmdShow )
{
  static char szAppname[150];
  WNDCLASS wndclass;
  hInstance = hInst;

  strcpy ( szAppname, SimulationTitle );

  if (!hPrevInstance)
  {
    wndclass.style = CS_HREDRAW | CS_VREDRAW | CS_BYTEALIGNCLIENT;
    wndclass.lpfnWndProc = WndProc;
    wndclass.cbClsExtra = 0;
    wndclass.cbWndExtra = 0;
    wndclass.hInstance = hInstance;
    wndclass.hIcon = LoadIcon(hInstance, "SIMUL");
    wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
    wndclass.hbrBackground = GetStockObject(WHITE_BRUSH);
    wndclass.lpszMenuName = "";
    wndclass.lpszClassName = szAppname;

    RegisterClass(&wndclass);
  }

  HMainWindow = CreateWindow( szAppname, 
                              szAppname,
                              MainWinStyle,
                              Window_X, Window_Y, MAIN_WIN_WIDTH, MAIN_WIN_HEIGHT,
                              NULL,
                              NULL,
                              hInstance, 
                              NULL );
                       
  ShowWindow ( HMainWindow, nCmdShow );
  UpdateWindow ( HMainWindow );

  HMainWinFont = (HFONT) GetStockObject (SYSTEM_FONT);
}

void ArrangeMainWindowVisibility (void)
{
  if ( !Visible )
    ShowWindow ( HMainWindow, SW_HIDE );
}


/* break ? */

void CheckUserBreak (void)
{
  MSG msg;
  int exit = 0;

  PeekMessage (&msg, HMainWindow, 0, 0, PM_REMOVE);

  if ( msg.message == WM_KEYDOWN )
    exit = 1;

  if ( msg.message == WM_LBUTTONDOWN )
    exit = 1;

  if ( msg.message == WM_RBUTTONDOWN )
    exit = 1;

  if (exit)
    if ( MessageBox ( NULL,
                      "Continue this simulation ?",
                      "Simulation Interrupted",
                      MB_YESNO | MB_ICONSTOP )              == IDNO )
      ExitCode = 1;

  RedrawWindow ( HMainWindow, NULL, NULL, RDW_UPDATENOW );
}

/* alive ? */

void StillAlive (void)
{
  /* there is a memory leak in here ! */
  /*
  RECT rect;
  HDC PaintDC;
  static HPEN hpen;
  static HBRUSH hbr;
  
  if ( !Visible ) return;

  hpen = CreatePen(PS_SOLID, 1, RGB(0, 0, 0));

  GetClientRect( HMainWindow, &rect );
  rect.top +=12;
  rect.right -=10;
  rect.left = rect.right - 20;
  rect.bottom = rect.top + 12;

  PaintDC = GetDC(HMainWindow);
  SelectObject(PaintDC, hpen);

  hbr = CreateSolidBrush( RGB( rand()%255, rand()%255, rand()%255) );

  SelectObject(PaintDC, hbr);
  Rectangle ( PaintDC, rect.left, rect.top, rect.right, rect.bottom );
  */
  return;
}

/* what's on the screen */

int paint ( HDC hdc )
{
  PaintNow ();
  return 1;
}

void PaintNow ( void )
{
  RECT rect;
  HDC PaintDC;
  static HPEN hpen;
  static HBRUSH hbr1;
  static HBRUSH hbr2;
  static int first_time = 1;
  int l1,l2;
  
  if ( !Visible ) return;

  GetClientRect( HMainWindow, &rect );
  rect.left +=10;
  rect.top +=10;
  rect.right -=10;
  rect.bottom -=10;

  PaintDC = GetDC(HMainWindow);
  SelectObject(PaintDC, HMainWinFont);

  switch ( DisplayStatus )
  {
    /* initialize */
    
    case 0:

      DrawText( PaintDC, "Initializing...", -1, &rect, DT_LEFT );
      break;

    /* progress */

    case 1:

      if (first_time)
      {
        DrawText( PaintDC, "                        \n                      ", 
                  -1, &rect, DT_LEFT );
        hpen = CreatePen(PS_SOLID, 2, RGB(0, 0, 0));
        hbr1 = CreateSolidBrush( RGB(0, 0, 255));
        hbr2 = CreateSolidBrush( RGB(0, 255, 255));
        first_time = 0;
      

        /* text */
        
        sprintf ( DoneBarTxt, "0 %% finished               \n\n"
                              "estimated time left = ??:??:??       \n"
                              "elapsed time = %s%d:%s%d:%s%d        ",
                              ElapsHours < 10 ? "0" : "", ElapsHours, 
                              ElapsMinutes < 10 ? "0" : "", ElapsMinutes,
                              ElapsSeconds < 10 ? "0" : "", ElapsSeconds );

      }
      else
      {

        sprintf ( DoneBarTxt, "%d %% finished               \n\n"
                              "estimated time left = %s%d:%s%d:%s%d        \n"
                              "elapsed time = %s%d:%s%d:%s%d        ",
                              Progress, 
                              Hours < 10 ? "0" : "", Hours, 
                              Minutes < 10 ? "0" : "", Minutes,
                              Seconds < 10 ? "0" : "", Seconds,
                              ElapsHours < 10 ? "0" : "", ElapsHours, 
                              ElapsMinutes < 10 ? "0" : "", ElapsMinutes,
                              ElapsSeconds < 10 ? "0" : "", ElapsSeconds );
      }

      PaintDC = GetDC(HMainWindow);
      SelectObject(PaintDC, HMainWinFont);

      DrawText( PaintDC, DoneBarTxt, -1, &rect, DT_LEFT );

      /* progress bar */

      l2 = rect.right;
      l1 = rect.left  + (rect.right - rect.left) * Progress / 100.0;

      SelectObject(PaintDC, hpen);
      SelectObject(PaintDC, hbr1);
      Rectangle ( PaintDC, rect.left, 90, l1, rect.bottom );

      SelectObject(PaintDC, hpen);
      SelectObject(PaintDC, hbr2);
      Rectangle ( PaintDC, l1, 90, l2, rect.bottom );
      
      break;
  }
}

/* boxes */

void STOP_Box (char* text_arg)
{
  MessageBox (NULL, text_arg, "Error Message", MB_OK | MB_ICONSTOP);
}

void OK_Box (char* text_arg)
{
  MessageBox (NULL, text_arg, "Message", MB_OK | MB_ICONINFORMATION);
}

int YN_Box (char* text_arg)
{
  return ( MessageBox (NULL, text_arg, "Message", MB_YESNO | MB_ICONQUESTION) );
}


void ShowExplanationinGUI ( void )
{
  MessageBox (NULL, EXPLANATION, "GREAT-ER Simulator - Help Window", 0 );
}

#endif
